#include <stdio.h>

#include <time.h>

#include <tos.h>

#include <string.h>
#include "mfp.h"
#include "dma.h"
#include "system.h"
/* ------------------------------------------ */



#define FALSE      0

#define TRUE       1



#define OK         0L           /* OK status */

#define ERROR     -1L           /* ERROR status (timeout) */

#define ERRORL    -2L           /* ERROR status (long timeout) */

#define EWRITF    -10           /* GEMDOS write error code */

#define EREADF    -11           /* GEMDOS read error code */

#define CRITRETRY 0x00010000L   /* RETRY return code */



#define NRETRIES   3            /* number of times to retry -1 */

#define MAX_UNITS  16           /* Max number of drives attached */

#define MAXSECTORS 254          /* Max # sectors for a DMA */

#define MAXRETRIES 1



/* Timing constants */



#define LTIMEOUT   600L         /* long-timeout 3 sec */

#define STIMEOUT    20L         /* short-timeout 100 msec */



/* RWABS flags */



#define RW_FLAG       0x01      /* flag for read/write */

#define MEDIACH_FLAG  0x02      /* flag for read/write with mediachange */

#define RETRY_FLAG    0x04      /* flag for read/write with retries */

#define PHYSOP_FLAG   0x08      /* flag for physical/logical read/write */
/* ------------------------------------------ */

/* ASCI Commands */



#define HD_TDR   0x00           /* Test Drive Ready */

#define HD_RS    0x03           /* Request Sense */

#define HD_FD    0x04           /* Format Drive */

#define HD_READ  0x08           /* Read */

#define HD_WRITE 0x0A           /* Write */

#define HD_SEEK  0x0B           /* Seek */

#define HD_MSEL  0x15           /* Mode Select */



typedef long (*func)();         /* pointer to function returning a long */





/* Logical Drive Info */



 struct hd_drv {

            short dev_addr;     /* physical unit ASCI address */

            long  part_start;   /* start sector logical unit */

               } ;



/* HD_LOW.C */



long  hwrite();          /* Write to physical sectors on hard disk */

long  hread();           /* Read physical sectors from hard disk */

long  endcmd();          /* Wait for end of ASCI command and get status */

long  setss();           /* Set ASCI drive, sector # and number of sectors */

long  hdone(void);           /* Restore DMA device to normal */

void  setdma();          /* Setup DMA base (transfer) address */

long  qdone(void);           /* Wait for command byte handshake */

long  fdone(void);           /* Wait for operation done handshake */

long  wait_dma_cmpl();   /* Wait for DMA interrupt with timeout */

/* ------------------------------------------ */



char buffOUT[512];

char buffIN[512];



int SetID(int old, int new);


int WriteSector(int Sector);

int ReadSector(int Sector);

long wait_dma_cmpl(unsigned long t_ticks);
long fdone();
long qdone();
void setdma(long addr);
long hdone();
long setss(char *buff);
long endcmd(short mode);

/* ------------------------------------------ */



int main(void)

{

	int old,new,res, new2;

	char c;
	void *OldSP;



	printf("%c%c",27,'E');	

	printf("%c%c >>> SET_ID TOOL FOR SATANDISK DEVICE <<< ",27,'p');

	printf("\n >>>     26. july 2006, by Jookie     <<< %c%c\n", 27, 'q');



	printf("\n\n!!! WARNING !!!");

	printf("\nThis tool should be used only with SatanDisk device.");

	printf("\nAnyhow, it should do no harm to any other device, ");

	printf("\nand only should return an error.");

	printf("\nYou need to know the CURRENT SatanDisk ACSI ID.");

	printf("\nIf you don't know that, HDDRIVER can tell you.");

	printf("\n\nIF YOU UNDESTAND THAT THIS WILL CHANGE");

	printf("\nTHE ID OF YOUR SATANDISK, PRESS 's' TO CONTINUE!");



	c = Bconin(2);



	if(c!='s')

		return 0;



	/* ---------------------- */

	OldSP = (void *) Super((void *)0);  			/* supervisor mode */ 
	/* ---------------------- */

	printf("\n\nPlease enter the CURRENT ID: ");
	
	while(1)
		{
		res = scanf("%d", &old);		/* get the old ID */

		if(res==1 && old>-1 && old<8)		/* if OK */
			break;
		}

	printf("Please enter the NEW     ID: ");

	while(1)
		{
		res = scanf("%d", &new);		/* get the old ID */

		if(res==1 && new>-1 && new<8)		/* if OK */
			break;
		}

	printf("\n");


	new2 = SetID(old, new);



	if(new2==0xfe)
		printf("\nError by isuing the SET_ID command!\n(probably no response from the device).");

	else if(new2==0xfd)
		printf("\nThe SatanDisk replied: something went wrong when setting ID!");
	else if(new2==new)
		printf("\nThe SatanDisk replied: THE NEW ID WAS SET!\nPlease reboot the machine...");
	else
		printf("\nI received some other (SCSI) error,\nthe device probably isn't SatanDisk.");
	


	printf("\n\nFinished, press any key to quit.");

	c = Bconin(2);


	Super(OldSP);  			/* user mode */ 


	return 0;

}



/* ------------------------------------------ */

int SetID(int old, int new)

{

 char buff[13];
 long  status;

 short dummy = 0;

 short oldID;

 oldID = (old<<5) | 0x1f;		/* ICD command with the correct ID */


 FLOCK = -1;                            /* disable FDC operations */

 DMA->MODE = NO_DMA | HDC;              /* write 1st byte (0) with A1 low */

 DMA->DATA = oldID;			/* ID byte */

 DMA->MODE = NO_DMA | HDC | A0;         /* A1 high again */

 strcpy(buff,"SatanDisk");		/* write the Magic string with ID */
 buff[9] = new;

 if (setss(buff) != OK) 		/* write bytes  to contrlr */

 {

    hdone();                            /* restore DMA device to normal */

    return(0xfe);

 }


 status = endcmd(DMA_WR | NO_DMA | HDC | A0); /* wait for DMA completion */

 hdone();                               /* restore DMA device to normal */

 return(status);

}



/****************************************************************************/

long endcmd(short mode)

{

 if (fdone() != OK)                   /* wait for operation done ack */

    return(ERRORL);

 DMA->MODE = mode;                    /* write mode word to mode register */

 return((long)(DMA->DATA & 0x00FF));  /* return completion byte */

}



/****************************************************************************/



long setss(char *buff)

{
 int i;

 /* wait for the correct transfer of 0th byte */


 DMA->MODE = NO_DMA | HDC | A0;          /* A0 line high again */	

 if (qdone() != OK)                      /* wait for ack */

    return(ERROR);


 /* transfer the rest of the bytes */

 for(i=0; i<10; i++)			/* send the whole buffer */
	{

 	DMA->DATA = buff[i];

 	DMA->MODE = NO_DMA | HDC | A0;         /* write drv | MSB sector */

 	
	if (qdone() != OK)                     /* wait for ack */

    		return(ERROR);
	}

 


 return(OK);                            /* everything OK, tell the caller */

}



/****************************************************************************/



long hdone(void)

{

 DMA->MODE = NO_DMA;        /* restore DMA mode register */

 FLOCK = 0;                 /* FDC operations may get going again */

 return((long)DMA->STATUS); /* read and return DMA status register */

}



/****************************************************************************/



void setdma(long addr)

{

 DMA->ADDR[LOW]  = (char)(addr);

 DMA->ADDR[MID]  = (char)(addr >> 8);

 DMA->ADDR[HIGH] = (char)(addr >> 16);

}



/****************************************************************************/



long qdone(void)

{

 return(wait_dma_cmpl(STIMEOUT));

}



/****************************************************************************/



long fdone(void)

{

 return(wait_dma_cmpl(LTIMEOUT));

}



/****************************************************************************/



long wait_dma_cmpl(unsigned long t_ticks)

{

 unsigned long to_count;



 to_count = t_ticks + HZ_200;   /* calc value timer must get to */



 do

 {

    if ( (MFP->GPIP & IO_DINT) == 0) /* Poll DMA IRQ interrupt */

         return(OK);                 /* got interrupt, then OK */

 }  while (HZ_200 <= to_count);      /* check timer */



 return(ERROR);                      /* no interrupt, and timer expired, */

                                     /* return ERROR status.             */

}



/****************************************************************************/



